import pandas as pd
import os

# ### the dataset locate function
def dataset_path(directory=None):
    if directory == None:
        dataset_dir = input(
            "\nEnter the UAH dataset directory (example: ~/Downloads/UAH-DRIVESET-v1), you can download it from here this link: \nhttp://www.robesafe.uah.es/personal/eduardo.romera/uah-driveset/#download \n\nyour input:"
        )
        if dataset_dir == None:
            print("\nno entry, loading default directory")
            PARENT_DIR = "~/Downloads/dataset/UAH-DRIVESET-v1"
        else:
            PARENT_DIR = dataset_dir
    else:
        PARENT_DIR = str(directory)
    print("\nthe selected dataset directory is:", PARENT_DIR)
    return PARENT_DIR


# ### the driver, driving state and road type selection function


def datablockpath_selection(driver=None, state=None, roadtype=None, selector=1):
    directory2 = None
    if state == None and roadtype == None and driver == None:
        seldriver = input(
            "\nenter the driver from list: \n 1.'D1' 2.'D2' 3.'D3' 4.'D4' 5.'D5' 6.'D6' \n\nyour choice:"
        )
        selstate = input(
            "\nenter state from list: \n 1.'normal' 2.'agressif' 3.'drowsy' \n\nyour choice:"
        )
        selroadtype = input(
            "\nenter road type from list: \n 1.'secondary' 2.'motorway' \n\nyour choice:"
        )
    else:
        seldriver = str(driver)
        selstate = str(state)
        selroadtype = str(roadtype)

    if seldriver == "1" or seldriver == "D1":
        if selstate == "1" or selstate == "normal":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D1/20151110175712-16km-D1-NORMAL1-SECONDARY"
                directory2 = "D1/20151110180824-16km-D1-NORMAL2-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D1/20151111123124-25km-D1-NORMAL-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "2" or selstate == "agressif":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D1/20151111134545-16km-D1-AGGRESSIVE-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D1/20151111125233-24km-D1-AGGRESSIVE-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "3" or selstate == "drowsy":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D1/20151111135612-13km-D1-DROWSY-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D1/20151111132348-25km-D1-DROWSY-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        else:
            print(
                "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
            )
            print("\n Reminder: we only have 3 driving states")
    elif seldriver == "2" or seldriver == "D2":
        if selstate == "1" or selstate == "normal":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D2/20151120160904-16km-D2-NORMAL1-SECONDARY"
                directory2 = "D2/20151120162105-17km-D2-NORMAL2-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D2/20151120131714-26km-D2-NORMAL-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "2" or selstate == "agressif":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D2/20151120163350-16km-D2-AGGRESSIVE-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D2/20151120133502-26km-D2-AGGRESSIVE-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "3" or selstate == "drowsy":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D2/20151120164606-16km-D2-DROWSY-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D2/20151120135152-25km-D2-DROWSY-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        else:
            print(
                "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
            )
            print("\n Reminder: we only have 3 driving states")
    elif seldriver == "3" or seldriver == "D3":
        if selstate == "1" or selstate == "normal":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D3/20151126124208-16km-D3-NORMAL1-SECONDARY"
                directory2 = "D3/20151126125458-16km-D3-NORMAL2-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D3/20151126110502-26km-D3-NORMAL-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "2" or selstate == "agressif":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D3/20151126130707-16km-D3-AGGRESSIVE-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D3/20151126134736-26km-D3-AGGRESSIVE-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "3" or selstate == "drowsy":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D3/20151126132013-17km-D3-DROWSY-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D3/20151126113754-26km-D3-DROWSY-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        else:
            print(
                "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
            )
            print("\n Reminder: we only have 3 driving states")
    elif seldriver == "4" or seldriver == "D4":
        if selstate == "1" or selstate == "normal":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D4/20151203171800-16km-D4-NORMAL1-SECONDARY"
                directory2 = "D4/20151203173103-17km-D4-NORMAL2-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D4/20151204152848-25km-D4-NORMAL-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "2" or selstate == "agressif":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D4/20151203174324-16km-D4-AGGRESSIVE-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D4/20151204154908-25km-D4-AGGRESSIVE-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "3" or selstate == "drowsy":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D4/20151203175637-17km-D4-DROWSY-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D4/20151204160823-25km-D4-DROWSY-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        else:
            print(
                "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
            )
            print("\n Reminder: we only have 3 driving states")
    elif seldriver == "5" or seldriver == "D5":
        if selstate == "1" or selstate == "normal":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D5/20151211162829-16km-D5-NORMAL1-SECONDARY"
                directory2 = "D5/20151211164124-17km-D5-NORMAL2-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D5/20151209151242-25km-D5-NORMAL-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "2" or selstate == "agressif":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D5/20151211165606-12km-D5-AGGRESSIVE-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D5/20151209153137-25km-D5-AGGRESSIVE-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "3" or selstate == "drowsy":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D5/20151211170502-16km-D5-DROWSY-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D5/20151211160213-25km-D5-DROWSY-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        else:
            print(
                "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '', \n you can also use the entry numbers "
            )
            print("\n Reminder: we only have 3 driving states")
    elif seldriver == "6" or seldriver == "D6":
        if selstate == "1" or selstate == "normal":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D6/20151221112434-17km-D6-NORMAL-SECONDARY"
                directory2 = ""
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D6/20151217162714-26km-D6-NORMAL-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '' , \nyou can also use the entry numbers "
                )
                print("\nReminder: we only have 2 road types")
        elif selstate == "2" or selstate == "agressif":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = ""
                print(
                    "\nATTENTION! driver 6 does not have data for state :agressif and road type: secondary"
                )
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D6/20151221120051-26km-D6-AGGRESSIVE-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '' , \nyou can also use the entry numbers "
                )
                print("\n Reminder: we only have 2 road types")
        elif selstate == "3" or selstate == "drowsy":
            if selroadtype == "1" or selroadtype == "secondary":
                directory = "D6/20151221113846-16km-D6-DROWSY-SECONDARY"
            elif selroadtype == "2" or selroadtype == "motorway":
                directory = "D6/20151217164730-25km-D6-DROWSY-MOTORWAY"
            else:
                print(
                    "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '' , \nyou can also use the entry numbers "
                )
                print("\nReminder: we only have 2 road types")
        else:
            print(
                "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '' , \nyou can also use the entry numbers "
            )
            print("\nReminder: we only have 3 driving states")
    else:
        directory = ""
        directory2 = ""
        print(
            "ERROR: No such entry type or misspelled entry type, try to write the entry correctly or remove the '' , \nyou can also use the entry numbers "
        )
        print("\nReminder: we only have 6 drivers")
    if selector == None:
        prompt = "We have two directories for this state and road type, choose which directory you want or you can choose to return both from the list: \n 1. 'directory1' 2.'directory2' 3.'both' \n\nyour choice:"
        selected = str(input(prompt))
    else:
        selected = str(selector)
    if (selected == "1" or selected == "directory 1") and directory != None:
        print("you choose the default first directory: \n")
        print("your directory is: ", directory)
        return directory
    elif (selected == "2" or selected == "directory2") and directory2 != None:
        print("you choose the second directory: \n")
        print("your directory is: ", directory2)
        return directory2
    elif (selected == "3" or selected == "both") and (
        directory != None and directory2 != None
    ):
        print("you choose to select both directories: \n")
        print("your directories are: \n", directory, "\n", directory2, "\n")
        return directory, directory2
    else:
        print(
            "\nERROR: wrong entry or misspelling, please change the entry or check spelling or remove the '' , \nyou can also use the entry numbers. \n"
        )
        print(
            "Note: for Driver 6, the driver have a unique session in the case of normal state and secondary road \n"
        )
        print("Empty path generated")
        directory = ""
        return directory


# ### the data type directories selection function


def datatype_selection(data=""):
    if data == "":
        prompt = "\nplease choose which type of data from this list: \n 1.'GPS' 2.'Accelerometer' 3.'lane detection' 4.'vehicle detection' \n 5.'open street map' 6.'lane change events' 7.'inertial events' \n 8.'semantics final' 9.'semantics online' \n\nyour choice:"
        datatype = input(prompt)
    else:
        datatype = str(data)
    if datatype == "GPS" or datatype == "1":
        file = "RAW_GPS.txt"
    elif datatype == "Accelerometer" or datatype == "2":
        file = "RAW_ACCELEROMETERS.txt"
    elif datatype == "lane detection" or datatype == "3":
        file = "PROC_LANE_DETECTION.txt"
    elif datatype == "vehicle detection" or datatype == "4":
        file = "PROC_VEHICLE_DETECTION.txt"
    elif datatype == "open street map" or datatype == "5":
        file = "PROC_OPENSTREETMAP_DATA.txt"
    elif datatype == "lane change events" or datatype == "6":
        file = "EVENTS_LIST_LANE_CHANGES.txt"
    elif datatype == "inertial events" or datatype == "7":
        file = "EVENTS_INERTIAL.txt"
    elif datatype == "semantics final" or datatype == "8":
        file = "SEMANTIC_FINAL.txt"
    elif datatype == "semantics online" or datatype == "9":
        file = "SEMANTIC_ONLINE.txt"
    else:
        print(
            "ERROR: No such data type or misspelling, try to write it correctly or remove the '', \nyou can also use the data type numbers"
        )
        print("\nEmpty data type generated")
        file = ""
    return file


# ### path generation


def path_generator(dataset_dir="", conditions_vector=[], datatype=""):
    path = ""
    if dataset_dir == "" and conditions_vector == [] and datatype == "":
        datasetpath = dataset_path()
        conditionspath = datablockpath_selection()
        if conditionspath == "":
            print("\nError in the conditions entry, empty path generated ")
            return path
        datatypepath = datatype_selection()
        if datatypepath == "":
            print("\nError in the data type entry, empty path generated ")
            return path
        path = os.path.join(datasetpath, conditionspath, datatypepath)
        print("\n your path is: ", path)
    else:
        datasetpath = dataset_path(dataset_dir)
        conditionspath = datablockpath_selection(
            conditions_vector[0],
            conditions_vector[1],
            conditions_vector[2],
            conditions_vector[3],
        )
        if conditionspath == "":
            print("\nError in the conditions entry, empty path generated ")
            return path
        datatypepath = datatype_selection(datatype)
        if datatypepath == "":
            print("\nError in the data type entry, empty path generated ")
            return path
        path = os.path.join(datasetpath, conditionspath, datatypepath)
        print("\n your path is: ", path)
    return path


# ### set up column headers function


def data_header_selector(selection=""):
    if selection == "":
        prompt = "\nplease choose which type of header from this list: \n 1.'GPS' 2.'Accelerometer' 3.'lane detection' 4.'vehicle detection' \n 5.'open street map' 6.'lane change events' 7.'inertial events' \n 8.'semantics final' 9.'semantics online' \n\n your choice: "
        header_type = input(prompt)
    else:
        header_type = str(selection)
    if header_type == "GPS" or header_type == "1":
        header_type = "GPS"
        data_headers = [
            "Timestamp (seconds)",
            "Speed (km/h)",
            "Latitude coordinate (degrees)",
            "Longitude coordinate (degrees)",
            "Altitude (meters)",
            "Vertical accuracy (degrees)",
            "Horizontal accuracy (degrees)",
            "Course (degrees)",
            "Difcourse: course variation (degrees)",
        ]
    elif header_type == "Acceleromter" or header_type == "2":
        header_type = "Acceleromter"
        data_headers = [
            "Timestamp (seconds)",
            "Boolean of system activated (1 if >50km/h)",
            "Acceleration in X (Gs)",
            "Acceleration in Y (Gs)",
            "Acceleration in Z (Gs)",
            "Acceleration in X filtered by KF (Gs)",
            "Acceleration in Y filtered by KF (Gs)",
            "Acceleration in Z filtered by KF (Gs)",
            "Roll (degrees)",
            "Pitch (degrees)",
            "Yaw (degrees)",
        ]
    elif header_type == "lane detection" or header_type == "3":
        header_type = "lane detection"
        data_headers = [
            "Timestamp (seconds)",
            "X: car position relative to lane center (meters)",
            "Phi: car angle relative to lane curvature (degrees)",
            "W: road width (meters)",
            "State of the lane det. algorithm [-1=calibrating,0=initializing, 1=undetected, 2=detected/running]",
        ]
    elif header_type == "vehicle detection" or header_type == "4":
        header_type = "vehicle detection"
        data_headers = [
            "Timestamp (seconds)",
            "Distance to ahead vehicle in current lane (meters) [value -1 means no car is detected in front]",
            "Time of impact to ahead vehicle (seconds) [distance related to own speed]",
            "Number of detected vehicles in this frame (traffic)",
            "GPS speed (km/h) [same as in RAW GPS]",
        ]
    elif header_type == "open street map" or header_type == "5":
        header_type = "open street map"
        data_headers = [
            "Timestamp (seconds)",
            "Maximum allowed speed of current road (km/h)",
            "Reliability of obtained maxspeed (0=unknown,1=reliable, 2=used previously obtained maxspeed,3=estimated by type of road)",
            "Type of road (motorway, trunk, secondary...)",
            "Number of lanes in current road",
            "Estimated current lane (1=right lane, 2=first left lane, 3=second left lane, etc) [experimental]",
            "GPS Latitude used to query OSM (degrees)",
            "GPS Longitude used to query OSM (degrees)",
            "OSM delay to answer query (seconds)",
            "GPS speed (km/h) [same as in RAW GPS]",
        ]
    elif header_type == "lane change events" or header_type == "6":
        header_type = "lane change events"
        data_headers = [
            "Timestamp (seconds)",
            "Type [+ indicates right and - left, 1 indicates normal lane change and 2 slow lane change]",
            "GPS Latitude of the event (degrees)",
            "GPS Longitude of the event (degrees)",
            "Duration of the lane change (seconds) [measured since the car position is near the lane marks]",
            "Time threshold to consider irregular change (secs.) [slow if change duration is over this threshold and fast if duration is lower than threshold/3]",
        ]
    elif header_type == "inertial events" or header_type == "7":
        header_type = "inertial events"
        data_headers = [
            "Timestamp (seconds)",
            "Type (1=braking, 2=turning, 3=acceleration)",
            "Level (1=low, 2=medium, 3=high)",
            "GPS Latitude of the event",
            "GPS Longitude of the event ",
            "Date of the event in YYYYMMDDhhmmss format",
        ]
    elif header_type == "semantics final" or header_type == "8":
        header_type = "semantics final"
        data_headers = [
            "Hour of route start",
            "Minute of route start",
            "Second of route start",
            "Average speed during trip (km/h)",
            "Maximum achieved speed during route (km/h)",
            "Lanex score (internal value, related to lane drifting)",
            "Driving time (in minutes)",
            "Hour of route end",
            "Minute of route end",
            "Second of route end",
            "Trip distance (km)",
            "ScoreLongDist	(internal value, Score accelerations)",
            "ScoreTranDist	(internal value, Score turnings)",
            "ScoreSpeedDist (internal value, Score brakings)",
            "ScoreGlobal (internal value, old score that is not used anymore)",
            "Alerts Long (internal value)",
            "Alerts Late (internal value)",
            "Alerts Lanex (internal value)",
            "Number of vehicle stops during route (experimental, related to fuel efficiency estimation)",
            "Speed variability (experimental, related to fuel efficiency estimation)",
            "Acceleration noise (experimental, related to fuel efficiency estimation)",
            "Kinetic energy (experimental, related to fuel efficiency estimation)",
            "Driving time (in seconds)",
            "Number of curves in the route",
            "Power exherted (experimental, related to fuel efficiency estimation)",
            "Acceleration events (internal value)",
            "Braking events (internal value)",
            "Turning events (internal value)",
            "Longitudinal-distraction Global Score (internal value, combines mean[31] and std[32])",
            "Transversal-distraction Global Score (internal value, combines mean[33] and std[34])",
            "Mean Long.-dist. score (internal value)",
            "STD Long.-dist. score (internal value)",
            "Average Trans.-dist. score (internal value)",
            "STD Trans.-dist. score (internal value)",
            "Lacc (number of low accelerations)",
            "Macc (number of medium accelerations)",
            "Hacc (number of high accelerations)",
            "Lbra (number of low brakings)",
            "Mbra (number of medium brakings)",
            "Hbra (number of high brakings)",
            "Ltur (number of low turnings)",
            "Mtur (number of medium turnings)",
            "Htur (number of high turnings)",
            "Score total (base 100, direct mean of the other 7 scores [45-51])",
            "Score accelerations (base 100)",
            "Score brakings (base 100)",
            "Score turnings (base 100)",
            "Score lane-weaving (base 100)",
            "Score lane-drifting (base 100)",
            "Score overspeeding (base 100)",
            "Score car-following (base 100)",
            "Ratio normal (base 1)",
            "Ratio drowsy (base 1)",
            "Ratio aggressive (base 1)",
        ]
    elif header_type == "semantics online" or header_type == "9":
        header_type = "semantics online"
        data_headers = [
            "TimeStamp since route start (seconds)",
            "GPS Latitude (degrees)",
            "GPS Longitude (degrees)",
            "Score total WINDOW (base 100, direct mean of the other 7 scores)",
            "Score accelerations WINDOW (base 100)",
            "Score brakings WINDOW (base 100)",
            "Score turnings WINDOW (base 100)",
            "Score weaving WINDOW (base 100)",
            "Score drifting WINDOW (base 100)",
            "Score overspeeding WINDOW (base 100)",
            "Score car-following WINDOW (base 100)",
            "Ratio normal WINDOW (base 1)",
            "Ratio drowsy WINDOW (base 1)",
            "Ratio aggressive WINDOW (base 1)",
            "Ratio distracted WINDOW (1=distraction detected in last 2 seconds, 0=otherwise)",
            "Score total (base 100, direct mean of the other 7 scores)",
            "Score accelerations (base 100)",
            "Score brakings (base 100)",
            "Score turnings (base 100)",
            "Score weaving (base 100)",
            "Score drifting (base 100)",
            "Score overspeeding (base 100)",
            "Score car-following (base 100)",
            "Ratio normal (base 1)",
            "Ratio drowsy (base 1)",
            "Ratio aggressive (base 1)",
            "Ratio distracted (1=distraction detected in last 2 seconds, 0=otherwise)",
        ]
    else:
        data_headers = []
        print(
            "ERROR: No such header type or header misspelled, try to write it correctly or remove the '', \nyou can also use the header numbers"
        )
        print("\nEmpty data header type generated\n")
    print("your data header type is : ", header_type)
    return data_headers


# ### data reader function


def data_reader(path=""):
    data = []
    if "SEMANTIC_FINAL" in path:
        delimiter = None
    else:
        delimiter = " "
    try:
        data = pd.read_table(path, header=None, delimiter=delimiter)
    except Exception:
        print(
            "\nERROR: please import numpy as pd so this function works and verify the entry path"
        )
        print("\nEmpty data table generated")
        return data
    if "GPS" in path:
        header = 1
        header_type = "GPS"
        data = data.drop([9, 10, 11, 12], axis=1)
    elif "ACCELEROMETER" in path:
        header = 2
        header_type = "Accelerometer"
        try:
            data = data.drop(11, axis=1)
        except Exception:
            data = data
    elif "LANE_DETECTION" in path:
        header = 3
        header_type = "lane detection"
    elif "VEHICLE" in path:
        header = 4
        header_type = "vehicle detection"
        data = data.drop(5, axis=1)
    elif "OPENSTREETMAP" in path:
        header = 5
        header_type = "open street map"
        data = data.drop(10, axis=1)
    elif "CHANGES" in path:
        header = 6
        header_type = "lane change events"
    elif "INERTIAL" in path:
        header = 7
        header_type = "inertial events"
        data = data.drop(6, axis=1)
    elif "FINAL" in path:
        header = 8
        header_type = "semantics final"
        data = data.transpose()
    elif "ONLINE" in path:
        header = 9
        header_type = "semantics online"
        data = data.drop(27, axis=1)
    data.columns = data_header_selector(header)
    print(
        "\033[1m",
        "        Data:",
        header_type,
        "\n",
        "******* READ SUCCESSFULLY *******",
        "\033[0m",
    )
    return data


# ### window stepping function


def window_stepping(data=[], window_size=0):
    segment = []
    final_data = pd.DataFrame()
    if len(data) != 0:
        if window_size == 0:
            final_data = data
            print("\nERROR:Please enter the window size")
            print("\nATTENTION: Entry data returned without window stepping")
            return final_data
        else:
            for i in range(0, len(data) - 1, window_size):
                segment = data[i : i + window_size]
                row = segment.mean()
                final_data = final_data.append(row, ignore_index=True)
    else:
        final_data = []
        print("ERROR: Empty data entry")
    return final_data


# ### merge dataframes functions


def merge_dataframes(data1=[], data2=[]):
    try:
        while data1.dtypes["Timestamp (seconds)"] != "int64":
            print(
                "\nWarning: Your data1 Timestamps are not integers, we are going to change their type\n"
            )
            data1["Timestamp (seconds)"] = data1["Timestamp (seconds)"].astype("int")
            print(
                "data1 timestamp type changed to : ",
                data1.dtypes["Timestamp (seconds)"],
                "\n",
            )
        while data2.dtypes["Timestamp (seconds)"] != "int64":
            print(
                "Warning: Your data2 Timestamps are not integers, we are going to change their type\n"
            )
            data2["Timestamp (seconds)"] = data2["Timestamp (seconds)"].astype("int")
            print(
                "data2 timestamp type changed to : ",
                data2.dtypes["Timestamp (seconds)"],
                "\n",
            )
        data_merged = data1.set_index("Timestamp (seconds)").join(
            data2.set_index("Timestamp (seconds)")
        )
        data_merged = data_merged.reset_index()
        print("\033[1m", "******* DATA SUCCESSFULLY MERGED *******", "\033[0m")
    except Exception:
        print(
            "ERROR: empty data entries or one data entry or both do not have 'Timestamp (seconds)' column, \nplease renter your two dataframes or check their columns before entry "
        )
        print("\nplease import pandas for this function to work")
        print("\nEmpty data returned")
        data_merged = []
    return data_merged


# ### interpolate_missing_data function


def interpolate_missing_data(data=[]):
    try:
        print("\n    State before interpolation    \n")
        print("COLUMNS                   ", "NUMBER OF RAWS WITH MISSING DATA")
        print(data.isnull().sum())
        missing_values = data.drop(["Timestamp (seconds)"], axis=1)
        missing_values = missing_values.interpolate(method="cubic", limit=3)
        data[missing_values.columns] = missing_values
        data_interpolated = data
    except Exception:
        data_interpolated = []
        print(data_interpolated)
        print("\nERROR: empty data entry or non dataframe type")
        print("\nplease import pandas for this function to work")
        print("\nEmpty data returned")

    try:
        print("\n    State after interpolation    \n")
        print("COLUMNS                   ", "NUMBER OF RAWS WITH MISSING DATA")
        print(data_interpolated.isnull().sum())
    except Exception:
        pass
    return data_interpolated


# ### removing incomplete raws functions


def removing_incomplete_raws(data=[]):
    try:
        data = data.dropna()
        data = data.reset_index(drop=True)
        print(data.count())
        print(
            "is there any missing data values? :",
            "\033[1m",
            data.isnull().values.any(),
            "\033[0m",
        )
    except Exception:
        print(
            "ERROR: empty data entry or non dataframe type, please enter your data dataframe"
        )
        print("\nplease import pandas for this function to work")
        print("\nEmpty data returned")
        data = []
    return data


# ## access any data skeleton function


def access_data(
    dataloc="",
    driver=1,
    state=1,
    road=1,
    selector=1,
    data_type1="",
    data_type2="",
    ws1=0,
    ws2=0,
):
    path1 = path_generator(dataloc, [driver, state, road, selector], data_type1)
    path2 = path_generator(dataloc, [driver, state, road, selector], data_type2)
    data1 = data_reader(path1)
    data2 = data_reader(path2)
    if ws1 != 0 and ws2 != 0:
        data = merge_dataframes(
            data1=window_stepping(data1, ws1), data2=window_stepping(data2, ws2)
        )
    elif ws1 != 0 and ws2 == 0:
        data = merge_dataframes(data1=window_stepping(data1, ws1), data2=data2)
    elif ws2 != 0 and ws1 == 0:
        data = merge_dataframes(data1=data1, data2=window_stepping(data2, ws2))
    else:
        print("ERROR: windowstep entry not acceptable, windowstep should be integer")
    data = interpolate_missing_data(data)
    data = removing_incomplete_raws(data)
    if state == 1:
        data["target"] = "normal"
    elif state == 2:
        data["target"] = "agressif"
    elif state == 3:
        data["target"] = "drowsy"
    if driver == 1:
        data["driver"] = "1"
    elif driver == 2:
        data["driver"] = "2"
    elif driver == 3:
        data["driver"] = "3"
    elif driver == 4:
        data["driver"] = "4"
    elif driver == 5:
        data["driver"] = "5"
    elif driver == 6:
        data["driver"] = "6"
    if road == 1:
        data["road"] = "secondary"
    elif road == 2:
        data["road"] = "motorway"
    return data



import pkg_resources
import pandas as pd
import os
from sklearn.preprocessing import LabelEncoder, OneHotEncoder
from sklearn.preprocessing import (
    StandardScaler,
    MinMaxScaler,
    MaxAbsScaler,
    RobustScaler,
    QuantileTransformer,
    PowerTransformer,
    Normalizer,
)


class DataPreprocessor:
    """[summary]
    A class for Data preprocessing specialized in time series data analysis from dataframes
    """

    def __init__(
        self,
        data_path="",
        specific_data=None,
        target_name="target",
        dataset=None,
        scaler="0",
        droped_columns=["Timestamp (seconds)"],
        no_encoding_columns=[],
        no_scaling_columns=["target"],
        window_size=0,
        step=0,
        average_window=False,
        from_csv=True,
        save_dataset=False,
        name_saved_dataset="Dataset",
        verbose=0,
    ):
        if dataset is None or isinstance(dataset, str):
            if from_csv is True:
                if dataset in [
                    "UAHdataset",
                    "uahdataset",
                    "UAHDataset",
                    "UAHDATASET",
                    "uah",
                    "UAH",
                ]:
                    self.raw_dataset = self.uahdataset_loading(
                        data_path, specific=specific_data, verbose=verbose
                    )
                else:
                    try:
                        self.raw_dataset = pd.read_csv(data_path)
                        if verbose == 1:
                            print(
                                "Reading data from provided path to the data csv file"
                            )
                    except Exception:
                        print("\nError reading data, verify the provided path")
        else:
            self.raw_dataset = dataset
            if verbose == 1:
                print("Reading from the dataset argement the provided dataframe")
        self.filtered_dataset = self.raw_dataset.drop(labels=droped_columns, axis=1)
        self.numeric_dataset = self.filtered_dataset.copy(deep=True)
        for column in self.numeric_dataset.columns:
            if (
                self.numeric_dataset.dtypes[column] != float
                and self.numeric_dataset.dtypes[column] != int
            ):
                if column in no_encoding_columns:
                    if verbose == 1:
                        print(
                            f"skipping \033[1m{column}\033[0m label encoding for being in the no_encoding_columns"
                        )
                else:
                    self.numeric_dataset = self.label_encoding(
                        self.numeric_dataset, target=column, verbose=verbose
                    )
        self.scaled_dataset, self.scaler_name = self.data_scaling(
            self.numeric_dataset,
            excluded_axis=no_scaling_columns,
            scaler=scaler,
            verbose=verbose,
        )
        self.ml_dataset = self.scaled_dataset
        self.features = self.scaled_dataset.drop(target_name, axis=1)
        self.target_column = self.scaled_dataset[target_name]
        self.target = self.one_hot_encoding(
            self.scaled_dataset, target=target_name, verbose=verbose
        )
        self.preprocessed_dataset = self.features.copy(deep=True)
        for i in self.target.columns:
            column_name = f"target {i}"
            self.preprocessed_dataset[column_name] = self.target[i]
        if window_size > 0:
            if verbose == 1:
                print(
                    "\n\033[1mThe window steeping can take some time depending on the dataset (for UAHDataset, it is between 1 to 5 minutes)\033[0m"
                )
            self.windowed_dataset = self.scaled_dataset.copy(deep=True)
            self.windowed_dataset = self.window_stepping(
                self.windowed_dataset,
                window_size=window_size,
                step=step,
                average_window=average_window,
                verbose=verbose,
            )
            self.windowed_ml_dataset = self.windowed_dataset
            self.windowed_features = self.windowed_dataset.drop(target_name, axis=1)
            self.windowed_target_column = self.windowed_dataset[target_name]
            self.windowed_target = self.one_hot_encoding(
                self.windowed_dataset, target=target_name, verbose=verbose
            )
            self.windowed_preprocessed_dataset = self.windowed_features.copy(deep=True)
            for i in self.windowed_target.columns:
                column_name = f"target {i}"
                self.windowed_preprocessed_dataset[column_name] = self.windowed_target[
                    i
                ]
        if save_dataset == True:
            NEW_PATH = "preprocessed_dataset"
            if not os.path.exists(NEW_PATH):
                os.makedirs(NEW_PATH)
            self.ml_dataset.to_csv(
                f"{NEW_PATH}/Preprocessed_ml_{name_saved_dataset}.csv",
                index=False,
            )
            self.preprocessed_dataset.to_csv(
                f"{NEW_PATH}/Preprocessed_{name_saved_dataset}.csv", index=False
            )
            if window_size > 0:
                self.windowed_ml_dataset.to_csv(
                    f"{NEW_PATH}/Windowed_preprocessed_ml_{name_saved_dataset}.csv",
                    index=False,
                )
                self.windowed_preprocessed_dataset.to_csv(
                    f"{NEW_PATH}/Windowed_preprocessed_{name_saved_dataset}.csv",
                    index=False,
                )
            if verbose == 1:
                print(
                    f"\n\033[1mPREPROCESSED DATASETS ARE SAVED IN THIS DIRECTORY : {os.getcwd()}/{NEW_PATH}/ \nWith the name similar to: Preprocessed_{name_saved_dataset}.csv  \033[0m\n"
                )

    @staticmethod
    def uahdataset_loading(path="", specific=None, verbose=1):
        if path == "":
            DATA_PATH = pkg_resources.resource_filename(
                __name__, "Datasets/UAH_dataset/dataset/UAHDataset.csv"
            )
            print(f"\nloading the internal \033[1mUAHDataset\033[0m from maaml\n")
            data = pd.read_csv(DATA_PATH)
        else:
            try:
                data = pd.read_csv(path)
                if verbose == 1:
                    print("\nUAHDataset read successfully\n")
            except Exception:
                print("\nERROR: bad path entry\nEmpty data variable returned")
                data = []
                return data
        if specific is None:
            data_info = "full data loaded successfully\n"
        elif str(specific) == "secondary road" or str(specific) == "":
            data = data.loc[data["road"] == "secondary"]
            data = data.drop("road", axis=1)
            data_info = "data of secondary road loaded successfully"
        elif str(specific) == "motorway road" or str(specific) == "0":
            data = data.loc[data["road"] == "motorway"]
            data = data.drop("road", axis=1)
            data_info = "data of motorway road loaded successfully"
        elif int(specific) < 7:
            data = data.loc[data["driver"] == int(specific)]
            data = data.drop("driver", axis=1)
            data_info = f"data of driver number {int(specific)} loaded successfully \n"
        else:
            print(
                "ERROR: wrong specific entry or specific entry does not exist\nEmpty data returned "
            )
            data = []
        if verbose == 1:
            print(data_info)
        return data

    @staticmethod
    def label_encoding(data, target, verbose=1):
        encoder = LabelEncoder()
        df = pd.DataFrame(data)
        try:
            if verbose == 1:
                print(
                    f"encoding the \033[1m{target}\033[0m column. The target labels are: {data[target].unique()} "
                )
            df[target] = encoder.fit_transform(data[target])
            if verbose == 1:
                print(f"The target labels after encoding : {df[target].unique()}")
        except Exception:
            print(
                f"ERROR: the column name '{target}' is not available in data\nno label encoding realized for this target\n"
            )
        return data

    @staticmethod
    def data_scaling(data, excluded_axis=[], scaler="minmax", verbose=1):
        scaled_df = data
        scaled_df = scaled_df.drop(excluded_axis, axis=1)
        columns_names_list = scaled_df.columns
        scaler = str(scaler)
        if scaler == "0" or scaler == "raw_data":
            scaler_name = "RawData (no scaling)"
            scaled_df = pd.DataFrame()
            for column in data.columns:
                scaled_df[column] = data[column].astype("float")
                scaled_df = scaled_df.reset_index(drop=True)
            scaled_df = scaled_df.fillna(0)
            if verbose == 1:
                print(f"data was not scaled, returned: {scaler_name}")
            return scaled_df, scaler_name
        elif scaler == "1" or scaler == "minmax":
            scalerfunction = MinMaxScaler()
            scaler_name = "MinMaxscaler"
        elif scaler == "2" or scaler == "standard":
            scalerfunction = StandardScaler()
            scaler_name = "Standardscaler"
        elif scaler == "3" or scaler == "maxabs":
            scalerfunction = MaxAbsScaler()
            scaler_name = "MaxAbsScaler"
        elif scaler == "4" or scaler == "robust":
            scalerfunction = RobustScaler()
            scaler_name = "RobustScaler"
        elif scaler == "5" or scaler == "quantile_normal":
            scalerfunction = QuantileTransformer(output_distribution="normal")
            scaler_name = "QuantileTransformer using normal distribution"
        elif scaler == "6" or scaler == "quantile_uniform":
            scalerfunction = QuantileTransformer(output_distribution="uniform")
            scaler_name = "QuantileTransformer using uniform distribution"
        elif scaler == "7" or scaler == "power_transform":
            scalerfunction = PowerTransformer(method="yeo-johnson")
            scaler_name = "PowerTransformer using the yeo-johnson method"
        elif scaler == "8" or scaler == "normalizer":
            scalerfunction = Normalizer()
            scaler_name = "Normalizer"
        else:
            print("\nERROR: wrong data entry or wrong scaler type\ninput data returned")
            scaler_name = "Worning : No scaling (something went wrong)"
            return data, scaler_name
        scaled_df = scalerfunction.fit_transform(scaled_df)
        scaled_df = pd.DataFrame(scaled_df, columns=columns_names_list)
        for i in excluded_axis:
            scaled_df[i] = data[i]
        scaled_df = scaled_df.fillna(0)
        if verbose == 1:
            print(f"data scaled with the {scaler_name}")
        return scaled_df, scaler_name

    @staticmethod
    def one_hot_encoding(data, target="target", verbose=1):
        encoder = OneHotEncoder()
        try:
            if verbose == 1:
                print(f"One Hot Encoder target: {data[target].unique()}")
            encoded = encoder.fit_transform(
                data[target].values.reshape(-1, 1)
            ).toarray()
        except Exception:
            try:
                if verbose == 1:
                    print(f"One Hot Encoder target: {data.unique()}")
                encoded = encoder.fit_transform(data.values.reshape(-1, 1)).toarray()
            except Exception:
                if verbose == 1:
                    print(
                        f"ERROR: target name '{target}' is not available in data\nNo One hot encoding realized"
                    )
                return data
        if verbose == 1:
            print(f"example of the target after One Hot encoding : {encoded[0]}")
        df = pd.DataFrame(encoded)
        return df

    @staticmethod
    def window_stepping(
        data=[], window_size=0, step=0, average_window=False, verbose=1
    ):
        segment = []
        final_data = pd.DataFrame()
        if len(data) != 0:
            if window_size == 0:
                final_data = data
                if verbose == 1:
                    print("\nATTENTION: Entry data returned without window stepping")
                return final_data
            else:
                if average_window is True:
                    if verbose == 1:
                        print("\nAverage window applied")
                    for i in range(0, len(data) - 1, step):
                        segment = data[i : i + window_size]
                        row = segment.mean()
                        final_data = final_data.append(row, ignore_index=True)
                else:
                    for i in range(0, len(data) - 1, step):
                        window = data[i : i + window_size]
                        final_data = final_data.append(window, ignore_index=True)
                    if verbose == 1:
                        print(
                            f"\nwindow stepping applied with window size: {window_size} and step : {step}"
                        )
        else:
            final_data = []
            print("ERROR: Empty data entry")
        return final_data


if __name__ == "__main__":
    preprocessor = DataPreprocessor(
        dataset="UAHdataset",
        no_encoding_columns=[],
        scaler=2,
        window_size=60,
        step=10,
        verbose=1,
        save_dataset=False,
    )
    print(f"\nthe raw dataset is: \n{preprocessor.raw_dataset}")
    print(f"\nthe dataset(after dropping columns) is\n{preprocessor.filtered_dataset}")
    print(f"the label encoded dataset: \n{preprocessor.numeric_dataset}")
    print(f"The used scaler is: {preprocessor.scaler_name}")
    print(f"\nthe scaled dataset is: \n{preprocessor.scaled_dataset}")
    print(f"\nthe dataset features are: \n{preprocessor.features}")
    print(f"\nthe dataset target column is: \n{preprocessor.target_column}")
    print(f"\nthe dataset one hot encoded target is: \n{preprocessor.target}")
    print(f"\nthe full preprocessed dataset is: \n{preprocessor.preprocessed_dataset}")
    print("\n ******* windowed data ******* \n")
    print(f"\nthe dataset windowed features are: \n{preprocessor.windowed_features}")
    print(
        f"\nthe dataset windowed target column is: \n{preprocessor.windowed_target_column}"
    )
    print(
        f"\nthe dataset windowed one hot encoded target is: \n{preprocessor.windowed_target}"
    )
    print(
        f"\nthe full windowed preprocessed dataset is: \n{preprocessor.windowed_preprocessed_dataset}"
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['maaml',
 'maaml.Datasets',
 'maaml.Datasets.UAH_dataset',
 'maaml.functions',
 'maaml.functions.Datasets']

package_data = \
{'': ['*'], 'maaml.Datasets.UAH_dataset': ['dataset/*']}

install_requires = \
['matplotlib>=3.4,<4.0',
 'numpy>=1.22.0,<2.0.0',
 'pandas>=1.3,<2.0',
 'sklearn>=0.0,<0.1',
 'tensorflow>=2.7.0,<3.0.0']

setup_kwargs = {
    'name': 'maaml',
    'version': '0.1.3',
    'description': 'For time series datasets Machine learning pipline',
    'long_description': None,
    'author': 'Najemeddine Abdennour',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from sklearn.tree import DecisionTreeClassifier, ExtraTreeClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.ensemble import RandomForestClassifier, ExtraTreesClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn import svm
from sklearn.linear_model import LogisticRegression
from sklearn.neural_network import MLPClassifier

from sklearn.model_selection import ShuffleSplit, train_test_split
from sklearn.preprocessing import label_binarize
from sklearn.metrics import (
    accuracy_score,
    precision_score,
    recall_score,
    f1_score,
    cohen_kappa_score,
    roc_auc_score,
    classification_report,
)
import numpy as np
import time
import os


class Evaluator:
    def __init__(
        self,
        model_name="4",
        paramater=None,
        features=None,
        target=None,
        dataset=None,
        target_name="target",
        nb_splits=5,
        test_size=0.3,
        full_eval=False,
        save_eval=False,
        save_tag=None,
        preprocessing_alias=None,
        verbose=0,
    ):
        if save_tag is None:
            save_tag = ""
        else:
            save_tag = f"_{save_tag}"
        self.model = i = 1
        target_name = [target_name]
        self.evaluation = []
        while True:
            if full_eval is False:
                self.model = self.model_building(
                    model_name=model_name, paramater=paramater, verbose=verbose
                )
            elif full_eval is True:
                try:
                    self.model = self.model_building(
                        model_name=i, paramater=paramater, verbose=verbose
                    )
                    i += 1
                except ValueError:
                    print("full evaluation complete")
                    if save_eval is True:
                        NEW_PATH = f"ML_EVALUATION{save_tag}"
                        if not os.path.exists(NEW_PATH):
                            os.makedirs(NEW_PATH)
                        if preprocessing_alias is not None:
                            np.savetxt(
                                f"{NEW_PATH}/{preprocessing_alias}_full_evaluation{save_tag}.csv",
                                self.evaluation,
                                delimiter=",",
                                fmt="%s",
                            )
                        else:
                            np.savetxt(
                                f"{NEW_PATH}/full_evaluation{save_tag}.csv",
                                self.evaluation,
                                delimiter=",",
                                fmt="%s",
                            )
                        if verbose == 1:
                            print(
                                f"full evaluation saved in:\n{os.getcwd()}/{NEW_PATH}/full_evaluation{save_tag}.csv"
                            )
                    break
            if "SVC" in str(self.model):
                self.model_name = (
                    str(self.model).replace("SVC", "SVMClassifier").replace("()", "")
                )
            else:
                self.model_name = str(self.model).replace("()", "")
            self.target_list = []
            if dataset is not None:
                for column_name in dataset.columns:
                    for keyname in target_name:
                        if keyname in column_name:
                            self.target_list.append(column_name)
            elif dataset is None:
                try:
                    if target.shape[1] > 1:
                        for name in range(0, target.shape[1]):
                            self.target_list.append(f"{target_name[0]} {name}")
                except IndexError:
                    self.target_list = target_name
                except Exception:
                    print("ERROR: Something went wrong in the entry target")
                    return
            else:
                print("ERROR: bad target name or bad target name entry ")
            self.cross_evaluation = self.model_cross_validating(
                features=features,
                target=target,
                dataset=dataset,
                target_names=self.target_list,
                nb_splits=nb_splits,
                test_size=test_size,
                preprocessing_alias=preprocessing_alias,
                verbose=verbose,
            )
            self.evaluation.extend(self.cross_evaluation)
            if save_eval is True and full_eval is False:
                NEW_PATH = f"ML_EVALUATION{save_tag}"
                if not os.path.exists(NEW_PATH):
                    os.makedirs(NEW_PATH)
                if preprocessing_alias is not None:
                    np.savetxt(
                        f"{NEW_PATH}/{preprocessing_alias}_{self.model_name}{save_tag}_evaluation.csv",
                        self.evaluation,
                        delimiter=",",
                        fmt="%s",
                    )
                else:
                    np.savetxt(
                        f"{NEW_PATH}/{self.model_name}{save_tag}_evaluation.csv",
                        self.evaluation,
                        delimiter=",",
                        fmt="%s",
                    )
                if verbose == 1:
                    print(
                        f"evaluation saved in:\n{os.getcwd()}/{NEW_PATH}/{self.model_name}{save_tag}_evaluation.csv"
                    )

            try:
                self.feature_importance_ranks = self.features_importance_ranking(
                    dataset=dataset,
                    classifier=self.model,
                    target_names=self.target_list,
                    features=features,
                    targets=target,
                    test_size=test_size,
                    verbose=verbose,
                )
            except Exception:
                if verbose == 1:
                    print(
                        f"The {str(self.model)} does not allow the extraction of feature importance ranks\nSkipping action"
                    )
            if full_eval is False:
                break

    @staticmethod
    def model_building(model_name="4", paramater=None, verbose=0):
        model_name = str(model_name)
        if model_name == "1" or model_name == "DecisionTree":
            model = DecisionTreeClassifier()
        elif model_name == "2" or model_name == "RandomForest":
            if paramater is not None:
                paramater = int(paramater)
                model = RandomForestClassifier(n_estimators=paramater)
            else:
                model = RandomForestClassifier()
        elif model_name == "3" or model_name == "ExtraTree":
            model = ExtraTreeClassifier()
        elif model_name == "4" or model_name == "ExtraTrees":
            if paramater is not None:
                paramater = int(paramater)
                model = ExtraTreesClassifier(n_estimators=paramater)
            else:
                model = ExtraTreesClassifier()
        elif model_name == "5" or model_name == "KNeighbors":
            if paramater is not None:
                paramater = int(paramater)
                model = KNeighborsClassifier(n_neighbors=paramater)
            else:
                model = KNeighborsClassifier()
        elif model_name == "6" or model_name == "GaussianNB":
            model = GaussianNB()
        elif model_name == "7" or model_name == "SVM":
            if paramater is not None:
                paramater = str(paramater)
                model = svm.SVC(gamma=paramater)
            else:
                model = svm.SVC()
        elif model_name == "8" or model_name == "LogisticRegression":
            if paramater is not None:
                paramater = str(paramater)
                model = LogisticRegression(
                    solver=paramater, multi_class="auto", max_iter=1000
                )
            else:
                model = LogisticRegression(multi_class="auto", max_iter=1000)
        elif model_name == "9" or model_name == "MLPClassifier":
            if paramater is not None:
                paramater = int(paramater)
                model = MLPClassifier(max_iter=paramater)
            else:
                model = MLPClassifier()
        elif int(model_name) > 9:
            raise ValueError(
                f"You entered {model_name}, a number bigger than the number of existant models"
            )
        else:
            print(
                "ERROR:wrong entry, this method have 9 diffrent classifiers, you could choose by number or by name"
            )
            model = "No model"
        if verbose == 1:
            print(f"\nThe {str(model)} is selected")
        return model

    def model_cross_validating(
        self,
        features=None,
        target=None,
        dataset=None,
        target_names=["target"],
        nb_splits=5,
        test_size=0.3,
        preprocessing_alias=None,
        verbose=0,
    ):
        start_time = time.perf_counter()
        if dataset is not None:
            X, Y = dataset.drop(target_names, axis=1), dataset[target_names]
        elif features is not None and target is not None:
            X, Y = features, target
        else:
            print(
                "ERROR: please enter a dataset with a target_name or enter features and target"
            )
            return
        cv = ShuffleSplit(n_splits=nb_splits, test_size=test_size, random_state=10)
        (
            acc_scores,
            pres_scores,
            rec_scores,
            f1,
            cokap_scores,
            roc_auc_scores,
            cv_scores,
        ) = ([], [], [], [], [], [], [])
        for train, test in cv.split(X, Y):
            if len(target_names) <= 1:
                classes = Y.unique()
                y_testb = label_binarize(Y[test], classes=classes)
            else:
                y_testb = Y.loc[test]
            Y_values = Y.values
            model = self.model
            try:
                pred = model.fit(X.loc[train], Y_values[train]).predict(X.loc[test])
            except ValueError:
                Y_values = Y.values.reshape(-1, 1).ravel()
                pred = model.fit(X.loc[train], Y_values[train]).predict(X.loc[test])
            acc_scores.append(
                accuracy_score(Y_values[test], pred, normalize=True) * 100
            )
            pres_scores.append(
                precision_score(Y_values[test], pred, average="macro") * 100
            )
            rec_scores.append(recall_score(Y_values[test], pred, average="macro") * 100)
            f1.append(f1_score(Y_values[test], pred, average="macro") * 100)
            cokap_scores.append(
                cohen_kappa_score(Y_values[test].reshape(-1, 1), pred.reshape(-1, 1))
                * 100
            )
            if len(target_names) <= 1:
                roc_auc_scores.append(roc_auc_score(y_testb, pred.reshape(-1, 1)) * 100)
            else:
                try:
                    roc_auc_scores.append(roc_auc_score(y_testb, pred) * 100)
                except ValueError:
                    roc_auc_scores.append(
                        roc_auc_score(Y_values[test].reshape(-1, 1), pred) * 100
                    )
        end_time = time.perf_counter()
        cv_scores = [
            ["MLclassifier", f"{self.model_name}"],
            ["execution time", f"{((end_time-start_time) / nb_splits): .2f} (s)"],
            ["accuracy", f"{np.mean(acc_scores):.2f}% (+/- {np.std(acc_scores):.2f}%)"],
            [
                "precision",
                f"{np.mean(pres_scores):.2f}% (+/- {np.std(pres_scores):.2f}%)",
            ],
            ["recall", f"{np.mean(rec_scores):.2f}% (+/- {np.std(rec_scores):.2f}%)"],
            ["F1", f"{np.mean(f1):.2f}% (+/- {np.std(f1):.2f}%)"],
            [
                "cohen_kappa",
                f"{np.mean(cokap_scores):.2f}% (+/- {np.std(cokap_scores):.2f}%)",
            ],
            [
                "roc_auc",
                f"{np.mean(roc_auc_scores):.2f}% (+/- {np.std(roc_auc_scores):.2f}%)",
            ],
        ]
        if preprocessing_alias is not None:
            cv_scores.insert(1, ["preprocessing", preprocessing_alias])
        if verbose == 1:
            for i, v in enumerate(cv_scores):
                if i == 0:
                    print(f"\033[1m\n{v[0]}:{v[1]}\033[0m")
                else:
                    print(f"cross validation {v[0]}: {v[1]}")
            print("\n")
        if verbose == 2:
            print(f"\nAccuracy evaluation for the separate splits:\n{acc_scores}")
            print(f"\nPrecision evaluation for the separate splits:\n{pres_scores}")
            print(f"\nRecall evaluation for the separate splits:\n{rec_scores}")
            print(f"\nF1 evaluation for the separate splits:\n{f1}")
            print(f"\nCohen_kappa evaluation for the separate splits:\n{cokap_scores}")
            print(f"\nRoc_Auc evaluation for the separate splits:\n{roc_auc_scores}")
        return cv_scores

    @staticmethod
    def features_importance_ranking(
        dataset=None,
        classifier=None,
        target_names=["target"],
        features=None,
        targets=None,
        test_size=0.3,
        verbose=0,
    ):
        if dataset is not None and target_names is not None:
            x = dataset.drop(target_names, axis=1)
            y = dataset[target_names].values
        elif dataset is None:
            x = features
            y = targets.values
        X_train, X_test, y_train, y_test = train_test_split(
            x, y, test_size=test_size, random_state=10
        )
        if classifier is None:
            model = ExtraTreesClassifier()
            classifier = str(model)
            print(
                f"The default Classifier for feature importance ranking is {classifier}"
            )
        else:
            model = classifier
        model = model.fit(X_train, y_train)
        pred = model.predict(X_test)

        if verbose == 1:
            f1score = f1_score(y_test, pred, average="macro")
            print(
                f"\nTrying to use {str(classifier)} for the feature ranking with an F1 score of : {f1score*100: .2f}%\n"
            )
        importances = model.feature_importances_
        ranks = x.T.drop(x.index, axis=1)
        ranks["importance %"] = importances * 100
        ranks = ranks.sort_values("importance %")[::-1]
        if verbose == 1:
            print(
                f"The {len(ranks)} features importance is ranked successfully using the {str(model)}"
            )
        return ranks

    @staticmethod
    def model_evaluating(
        dataset=None,
        classifier=None,
        target_names=["target"],
        features=None,
        targets=None,
        test_size=0.3,
        verbose=0,
    ):
        if dataset is not None and target_names is not None:
            x = dataset.drop(target_names, axis=1)
            y = dataset[target_names].values
        elif dataset is None:
            x = features
            y = targets.values
        else:
            print("No data is provided for the evaluation")
        X_train, X_test, y_train, y_test = train_test_split(
            x, y, test_size=test_size, random_state=10
        )
        if classifier is None:
            model = ExtraTreesClassifier()
            classifier = str(model)
            print(
                f"You did not provide a classifier, the default Classifier is {classifier}"
            )
        else:
            model = classifier
        model = model.fit(X_train, y_train)
        pred = model.predict(X_test)
        results = classification_report(y_test, pred)
        if verbose == 1:
            print(results)
        return results


def main():
    from maaml.preprocessing import DataPreprocessor as dp

    processed = dp(dataset="UAHdataset", scaler=2)
    uahdataset = processed.preprocessed_dataset
    alias = processed.scaler_name
    features = processed.features
    target_column = processed.target_column
    target = processed.target
    ml_evaluation = Evaluator(
        3,
        dataset=uahdataset,
        verbose=1,
        preprocessing_alias=alias,
        full_eval=True,
        save_eval=True,
    )
    print("\nThe target list is :", ml_evaluation.target_list)
    print("feature importance : \n", ml_evaluation.feature_importance_ranks)


if __name__ == "__main__":
    main()



import pandas as pd
from sklearn.preprocessing import (
    StandardScaler,
    MinMaxScaler,
    MaxAbsScaler,
    RobustScaler,
    QuantileTransformer,
    PowerTransformer,
    Normalizer,
)
from sklearn.preprocessing import LabelEncoder, OneHotEncoder

# #  dataset loader function
def dataset_loader(path="", full=True, specific="", verbose=1):
    if path == "":
        print("\nset path to default:", "\ndataset build/UAHdataset.csv\n")
        data = pd.read_csv("dataset build/UAHdataset.csv")
    else:
        try:
            data = pd.read_csv(path)
        except Exception:
            print("\nERROR: bad path entry", "\nReturn empty data")
            print("\nNote: pandas must be imported as pd")
            data = []
            return data
    while full == True:
        data_name = "full data loaded"
        return data
    specific = str(specific)
    if specific == "secondary road" or specific == "":
        data = data.loc[data["road"] == "secondary"]
        data = data.drop("road", axis=1)
        data_name = "secondary road data loaded"
    elif specific == "motorway road" or specific == "0":
        data = data.loc[data["road"] == "motorway"]
        data = data.drop("road", axis=1)
        data_name = "motorway road data loaded"
    elif int(specific) < 7:
        data = data.loc[data["driver"] == int(specific)]
        data = data.drop("driver", axis=1)
        data_name = "driver specific data loaded" + "\ndriver number:" + specific + "\n"
    else:
        print(
            "ERROR: wrong specific entry or specific entry does not exist",
            "\nEmpty data returned ",
        )
        print(
            'Note: Pandas library needed for this function to work and must be loaded as "pd"'
        )
        data = []
    if verbose == 1:
        print(data_name)
    return data


# # label encoding function
def encoding_label(data, target, verbose=1):
    encoder = LabelEncoder()
    df = pd.DataFrame(data)
    try:
        if verbose == 1:
            print("label encoder target: ", data[target].unique())
        df[target] = encoder.fit_transform(data[target])
        if verbose == 1:
            print("target after label encoding : ", df[target].unique())
    except Exception:
        if verbose == 1:
            print(
                f"ERROR: target name '{target}' is not available in data\n",
                f"no label encoding realized for '{target}'\n",
            )
        return data
    return df


# # data normalization function
def sklearn_data_normalizer(data, excluded_axis=[], scaler="minmax", verbose=1):
    normalized = data
    for i in excluded_axis:
        normalized = normalized.drop(i, axis=1)
    reserved_columns = normalized.columns
    scaler = str(scaler)
    if scaler == "0" or scaler == "raw_data":
        normalizer_name = "RawData (no scaling)"
        normalized = pd.DataFrame()
        for column in data.columns:
            normalized[column] = data[column].astype("float")
            normalized = normalized.reset_index(drop=True)
        normalized = normalized.fillna(0)
        if verbose == 1:
            print("data is not normalized, returned:", normalizer_name)
        return normalized, normalizer_name
    elif scaler == "1" or scaler == "minmax":
        scalerfunction = MinMaxScaler()
        normalizer_name = "MinMaxscaler"
    elif scaler == "2" or scaler == "standard":
        scalerfunction = StandardScaler()
        normalizer_name = "Standardscaler"
    elif scaler == "3" or scaler == "maxabs":
        scalerfunction = MaxAbsScaler()
        normalizer_name = "MaxAbsScaler"
    elif scaler == "4" or scaler == "robust":
        scalerfunction = RobustScaler()
        normalizer_name = "RobustScaler"
    elif scaler == "5" or scaler == "quantile_normal":
        scalerfunction = QuantileTransformer(output_distribution="normal")
        normalizer_name = "QuantileTransformer using normal distribution"
    elif scaler == "6" or scaler == "quantile_uniform":
        scalerfunction = QuantileTransformer(output_distribution="uniform")
        normalizer_name = "QuantileTransformer using uniform distribution"
    elif scaler == "7" or scaler == "power_transform":
        scalerfunction = PowerTransformer(method="yeo-johnson")
        normalizer_name = "PowerTransformer using method yeo-johnson"
    elif scaler == "8" or scaler == "normalizer":
        scalerfunction = Normalizer()
        normalizer_name = " sklearn Normalizer"
    else:
        print("\nERROR: wrong entry or wrong scaler type")
        print("\nreturned input data")
        return data
    if verbose == 1:
        print("data normalized with the", normalizer_name)
    normalized = scalerfunction.fit_transform(normalized)
    normalized = pd.DataFrame(normalized, columns=reserved_columns)
    for i in excluded_axis:
        normalized[i] = data[i]
    normalized = normalized.fillna(0)
    return normalized, normalizer_name


# # One hot encoder function
def One_hot_encoder(data, target="target", verbose=1):
    encoder = OneHotEncoder()
    try:
        if verbose == 1:
            print("One Hot Encoder target: ", data[target].unique())
        encoded = encoder.fit_transform(data[target].values.reshape(-1, 1)).toarray()
        if verbose == 1:
            print("example of target after One Hot encoding : ", encoded[0])
    except Exception:
        try:
            print("One Hot Encoder target: ", data.unique())
            encoded = encoder.fit_transform(data.values.reshape(-1, 1)).toarray()
            if verbose == 1:
                print("example of target after One Hot encoding : ", encoded[0])
        except Exception:
            if verbose == 1:
                print(
                    f"ERROR: target name '{target}' is not available in data\n",
                    f"no One hot encoding realized for '{target}'\n",
                )
            return data
    df = pd.DataFrame(encoded)
    return df
